import 'dart:io';
import 'package:path/path.dart' as p;

class FileDjUp {
  final String path;
  final String name;
  final String extension;
  final String djPostFix;
  final Map<String, String> widgetNameMap;
  final List<String> widgetImportLines;
  FileDjUp({
    required this.path,
    required this.name,
    this.extension = 'dart',
    this.djPostFix = 'up_dj_generated',
    this.widgetNameMap = const {},
    this.widgetImportLines = const [],
  });

  void generate() {
    var fileName = '$name.$extension';
    var absoluteFileName = p.join(path, fileName);

    var djFileName = '${name}_$djPostFix.$extension';
    var djAbsoluteFileName = p.join(path, djFileName);

    print('generating dj version of file $fileName');

    var fileHandler = File(absoluteFileName);
    var djFileHandler = File(djAbsoluteFileName);

    // Read Source file
    var fileLines = fileHandler.readAsLinesSync();

    // Process all lines
    var djFileLines = fileLines.map((line) => _processLine(line)).toList();

    // Add dart dj import lines
    djFileLines =
        ["import 'package:dj/dj.dart';"] + widgetImportLines + djFileLines;

    // create crossponding dj files
    djFileHandler.createSync();

    // write processed lines
    var fileWritter = djFileHandler.openWrite();
    djFileLines.forEach((line) => fileWritter.writeln(line));
    fileWritter.close();

    print('generated $djFileName');
  }

  String _processLine(String line) {
    var pLine = line;

    widgetNameMap.forEach((widgetName, djWidgetName) {
      if (pLine.contains(widgetName + '(')) {
        pLine = pLine.replaceAll(widgetName + '(', djWidgetName + '(');
      }
    });

    return pLine;
  }
}
